﻿'use strict';

// view model
function ChatMessengerSessionViewModel(userProviderKey, sessionId, mediaParams) {
    ChatSessionViewModel.call(this, userProviderKey, sessionId, mediaParams);
    var self = this;

    self.messagesLoading = ko.observable(false);
    self.expressionToSearch = ko.observable();
    self.messageOccurencePosition = ko.observable(0);
    self.messagesOccurencesIds = ko.observable(null);
    self.messageOccurrenceCluster = ko.observableArray(null);
    self.lastMessageOccurrenceIndexInCluster = null;

    self.resetSearching = function () { self.getResetSearching.call(self); };
    self.searchMessages = function () { self.getSearchMessages.call(self); };
    self.changeMessageCluster = function (next) { self.getChangeMessageCluster.call(self, next); };
    self.loadTresholdMessages = function (messagesObservable, next) { self.loadTresholdMessagesGetter.call(self, messagesObservable, next); };
}

ChatMessengerSessionViewModel.prototype = Object.create(ChatSessionViewModel.prototype);


ChatMessengerSessionViewModel.prototype.getResetSearching = function () {
    var self = this;

    self.messageOccurencePosition(0);
    self.messagesOccurencesIds(null);
    self.messageOccurrenceCluster(null);
    self.lastMessageOccurrenceIndexInCluster = null;
    self.expressionToSearch(null);
};

ChatMessengerSessionViewModel.prototype.getSearchMessages = function () {
    var self = this;

    self.messagesLoading(true);
    self.messageOccurencePosition(0);
    self.messagesOccurencesIds(null);
    self.messageOccurrenceCluster(null);
    self.lastMessageOccurrenceIndexInCluster = null;
    $.getJSON(RQ_URL + "/Chat/SearchMessages/", {
        sessionId: self.sessionId,
        searchExpression: self.expressionToSearch()
    }).done(function (res) {
        if (res) {
            self.messageOccurencePosition(res.MessagesOccurencesIds.length);
            self.messagesOccurencesIds(res.MessagesOccurencesIds);
            if (res.LastMessageOccurrenceCluster) {
                self.messageOccurrenceCluster(res.LastMessageOccurrenceCluster.Messages);
                self.lastMessageOccurrenceIndexInCluster = res.LastMessageOccurrenceCluster.MessageOccurrenceIndex;
            }
        }
    }).fail(function (error) {
        console.error('search: ' + error);
    }).always(function () {
        self.messagesLoading(false);
    });
};

ChatMessengerSessionViewModel.prototype.getChangeMessageCluster = function (next) {
    var self = this;

    if (!self.messagesOccurencesIds())
        return;

    let position = next && self.messagesOccurencesIds().length > self.messageOccurencePosition() ? self.messageOccurencePosition() + 1 : !next && self.messageOccurencePosition() > 1 ? self.messageOccurencePosition() - 1 : null;
    if (position) {
        self.messagesLoading(true);
        self.messageOccurrenceCluster(null);
        self.lastMessageOccurrenceIndexInCluster = null;
        $.getJSON(RQ_URL + "/Chat/LoadMessageCluster/", {
            sessionId: self.sessionId,
            messageId: self.messagesOccurencesIds()[position - 1]
        }).done(function (res) {
            self.messageOccurencePosition(position);
            if (res) {
                self.messageOccurrenceCluster(res.Messages);
                self.lastMessageOccurrenceIndexInCluster = res.MessageOccurrenceIndex;
            }
        }).fail(function (error) {
            console.error('getMessageCluster: ' + error);
        }).always(function () {
            self.messagesLoading(false);
        });
    }
};

ChatMessengerSessionViewModel.prototype.loadTresholdMessagesGetter = function (messagesObservable, next) {
    var self = this;

    if (!messagesObservable() || messagesObservable().length === 0)
        return;

    let idTreshold = messagesObservable()[next ? messagesObservable().length - 1 : 0].Id;
    if (idTreshold > 1) {
        self.messagesLoading(true);
        $.getJSON(RQ_URL + "/Chat/LoadTresholdMessages/", {
            sessionId: self.sessionId,
            idTreshold: idTreshold,
            next: next
        }).done(function (loadedMessages) {
            if (loadedMessages) {
                if (!next)
                    messagesObservable().splice.apply(messagesObservable(), [0, 0].concat(loadedMessages));
                else
                    messagesObservable().push.apply(messagesObservable(), loadedMessages);
            }
        }).fail(function (error) {
            console.warn('loadMessages: ' + error);
        }).always(function () {
            self.messagesLoading(false);
        });
    }
};

ChatMessengerSessionViewModel.prototype._oppositeCallingUserSessionGetter = function () {
    var self = this;
    return self.userSessions().length === 1 ? self.userSessions()[0].us : null;
};

ChatMessengerSessionViewModel.prototype._chatMediaSessionParamsGetter = function (automationMode, useCamera) {
    var self = this;

    return {
        pickUpMode: automationMode === chatUtils.CallAutomationModeEnum.PICK_UP ? useCamera ? "cam" : "mic" : null,
        callMode: automationMode === chatUtils.CallAutomationModeEnum.CALL ? useCamera ? "cam" : "mic" : null,
        sessionId: self.sessionId
    };
};