﻿'use strict';

var signalRConnection = (function () {
    function startConnection(initCallbacks, queryStrings) {

        function connectAfterTimeout() {
            if (SIGNALR_REPEAT_TIMEOUT === 0)
                return;

            setTimeout(function () {
                if ($.connection.hub.state === $.signalR.connectionState.disconnected) {
                    $.connection.hub.start().done(connectionDone);
                }
            }, SIGNALR_REPEAT_TIMEOUT);
        }

        function connectionDone() {
            $('#disconnection').hide();
            runInitCallbacks();
        }

        function runInitCallbacks() {
            if (initCallbacks)
            {
                for (let i = 0; i < initCallbacks.length; i++) {
                    if (typeof initCallbacks[i] === 'function') {
                        runInitCallback(initCallbacks[i]);
                    }
                }
            }
        }

        function runInitCallback(initCallback) {
            initCallback().catch(function (error) {
                if (SIGNALR_REPEAT_TIMEOUT === 0)
                    return;

                setTimeout(function () {
                    runInitCallback(initCallback);
                }, SIGNALR_REPEAT_TIMEOUT);
            });
        }

        $.connection.hub.reconnecting(function () {
            //$('#disconnection').show();
        });

        $.connection.hub.reconnected(function () {
            $('#disconnection').hide();
        });

        $.connection.hub.disconnected(function () {
            setTimeout(function () {
                $('#disconnection').show();
            }, 5000);
            connectAfterTimeout();
        });

        $.connection.hub.qs = {
            'ticketId': (typeof ticketId === 'undefined' ? null : ticketId)
        };

        if (queryStrings)
        {
            $(queryStrings).each(function (index, queryString) {
                for (let prop in queryString) {
                    $.connection.hub.qs[prop] = queryString[prop];
                }
            });
        }

        $.connection.hub.start().done(connectionDone);
    }

    return {
        startConnection: startConnection
    };
})();